@@ -18,7 +18,6 @@ import datetime
 import functools
 import iso8601
 import operator
-import six
 import threading
 import time
 import types  # noqa
@@ -1633,6 +1632,16 @@ class ComputeCloudMixin(_normalize.Normalizer):
 
         :raises: OpenStackCloudException on operation error.
         """
+        if (
+            isinstance(name_or_id, (str, bytes))
+            and not name_or_id.isdigit()
+        ):
+            aggregate = self.get_aggregate(name_or_id)
+            if not aggregate:
+                self.log.debug(
+                    "Aggregate %s not found for deleting", name_or_id)
+                return False
+            name_or_id = aggregate.id
         try:
             self.compute.delete_aggregate(name_or_id, ignore_missing=False)
             return True
@@ -1803,7 +1812,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
         proj = self.get_project(name_or_id)
         if not proj:
             raise exc.OpenStackCloudException(
-                "project does not exist: {}".format(name=proj.id))
+                "project does not exist: {name}".format(name=proj.id))
 
         data = proxy._json_response(
             self.compute.get(
@@ -1818,9 +1827,9 @@ class ComputeCloudMixin(_normalize.Normalizer):
         if hasattr(userdata, 'read'):
             userdata = userdata.read()
 
-        if not isinstance(userdata, six.binary_type):
+        if not isinstance(userdata, bytes):
             # If the userdata passed in is bytes, just send it unmodified
-            if not isinstance(userdata, six.string_types):
+            if not isinstance(userdata, str):
                 raise TypeError("%s can't be encoded" % type(userdata))
             # If it's not bytes, make it bytes
             userdata = userdata.encode('utf-8', 'strict')
