@@ -196,7 +196,7 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
     vm_state = resource.Body('OS-EXT-STS:vm_state')
 
     def _prepare_request(self, requires_id=True, prepend_key=True,
-                         base_path=None):
+                         base_path=None, **kwargs):
         request = super(Server, self)._prepare_request(requires_id=requires_id,
                                                        prepend_key=prepend_key,
                                                        base_path=base_path)
@@ -260,18 +260,20 @@ class Server(resource.Resource, metadata.MetadataMixin, resource.TagMixin):
         body = {'forceDelete': None}
         self._action(session, body)
 
-    def rebuild(self, session, name, admin_password,
+    def rebuild(self, session, name=None, admin_password=None,
                 preserve_ephemeral=False, image=None,
                 access_ipv4=None, access_ipv6=None,
                 metadata=None, personality=None):
         """Rebuild the server with the given arguments."""
         action = {
-            'name': name,
-            'adminPass': admin_password,
             'preserve_ephemeral': preserve_ephemeral
         }
         if image is not None:
             action['imageRef'] = resource.Resource._get_id(image)
+        if name is not None:
+            action['name'] = name
+        if admin_password is not None:
+            action['adminPass'] = admin_password
         if access_ipv4 is not None:
             action['accessIPv4'] = access_ipv4
         if access_ipv6 is not None:
