@@ -17,6 +17,7 @@
 import argparse as argparse_mod
 import collections
 import copy
+import errno
 import json
 import os
 import re
@@ -127,7 +128,7 @@ def _fix_argv(argv):
                 options=','.join(overlap)))
 
 
-class OpenStackConfig(object):
+class OpenStackConfig:
 
     # These two attribute are to allow os-client-config to plumb in its
     # local versions for backwards compat.
@@ -288,25 +289,28 @@ class OpenStackConfig(object):
 
             influxdb_cfg = metrics_config.get('influxdb', {})
             # Parse InfluxDB configuration
-            if influxdb_config:
-                influxdb_cfg.update(influxdb_config)
-            if influxdb_cfg:
-                config = {}
-                if 'use_udp' in influxdb_cfg:
-                    use_udp = influxdb_cfg['use_udp']
-                    if isinstance(use_udp, str):
-                        use_udp = use_udp.lower() in ('true', 'yes', '1')
-                    elif not isinstance(use_udp, bool):
-                        use_udp = False
-                        self.log.warning('InfluxDB.use_udp value type is not '
-                                         'supported. Use one of '
-                                         '[true|false|yes|no|1|0]')
-                    config['use_udp'] = use_udp
-                for key in ['host', 'port', 'username', 'password', 'database',
-                            'measurement', 'timeout']:
-                    if key in influxdb_cfg:
-                        config[key] = influxdb_cfg[key]
-                self._influxdb_config = config
+            if not influxdb_config:
+                influxdb_config = influxdb_cfg
+            else:
+                influxdb_config.update(influxdb_cfg)
+
+        if influxdb_config:
+            config = {}
+            if 'use_udp' in influxdb_config:
+                use_udp = influxdb_config['use_udp']
+                if isinstance(use_udp, str):
+                    use_udp = use_udp.lower() in ('true', 'yes', '1')
+                elif not isinstance(use_udp, bool):
+                    use_udp = False
+                    self.log.warning('InfluxDB.use_udp value type is not '
+                                     'supported. Use one of '
+                                     '[true|false|yes|no|1|0]')
+                config['use_udp'] = use_udp
+            for key in ['host', 'port', 'username', 'password', 'database',
+                        'measurement', 'timeout']:
+                if key in influxdb_config:
+                    config[key] = influxdb_config[key]
+            self._influxdb_config = config
 
         if load_envvars:
             statsd_host = statsd_host or os.environ.get('STATSD_HOST')
@@ -378,11 +382,17 @@ class OpenStackConfig(object):
     def _load_yaml_json_file(self, filelist):
         for path in filelist:
             if os.path.exists(path):
-                with open(path, 'r') as f:
-                    if path.endswith('json'):
-                        return path, json.load(f)
-                    else:
-                        return path, yaml.safe_load(f)
+                try:
+                    with open(path, 'r') as f:
+                        if path.endswith('json'):
+                            return path, json.load(f)
+                        else:
+                            return path, yaml.safe_load(f)
+                except IOError as e:
+                    if e.errno == errno.EACCES:
+                        # Can't access file so let's continue to the next
+                        # file
+                        continue
         return (None, {})
 
     def _expand_region_name(self, region_name):
@@ -772,8 +782,8 @@ class OpenStackConfig(object):
             # legacy clients have un-prefixed api-version options
             parser.add_argument(
                 '--{service_key}-api-version'.format(
-                    service_key=service_key.replace('_', '-'),
-                    help=argparse_mod.SUPPRESS))
+                    service_key=service_key.replace('_', '-')),
+                help=argparse_mod.SUPPRESS)
             adapter.register_service_adapter_argparse_arguments(
                 parser, service_type=service_key)
 
