@@ -131,7 +131,7 @@ construct a Connection with the ``CONF`` object and an authenticated Session.
 
     conn = connection.Connection(
         session=session,
-        oslo_config=CONF)
+        oslo_conf=CONF)
 
 From existing CloudRegion
 -------------------------
@@ -179,9 +179,9 @@ Additional information about the services can be found in the
 import warnings
 import weakref
 
+import concurrent.futures
 import keystoneauth1.exceptions
 import requestsexceptions
-import six
 
 from openstack import _log
 from openstack import _services_mixin
@@ -278,6 +278,7 @@ class Connection(
                  service_types=None,
                  global_request_id=None,
                  strict_proxies=False,
+                 pool_executor=None,
                  **kwargs):
         """Create a connection to a cloud.
 
@@ -345,6 +346,11 @@ class Connection(
             where it can be expected that the deployer config is correct and
             errors should be reported immediately.
             Default false.
+        :param pool_executor:
+        :type pool_executor: :class:`~futurist.Executor`
+            A futurist ``Executor`` object to be used for concurrent background
+            activities. Defaults to None in which case a ThreadPoolExecutor
+            will be created if needed.
         :param kwargs: If a config is not provided, the rest of the parameters
             provided are assumed to be arguments to be passed to the
             CloudRegion constructor.
@@ -380,7 +386,7 @@ class Connection(
 
         self._session = None
         self._proxies = {}
-        self.__pool_executor = None
+        self.__pool_executor = pool_executor
         self._global_request_id = global_request_id
         self.use_direct_get = use_direct_get
         self.strict_mode = strict
@@ -429,6 +435,14 @@ class Connection(
                 self.log.warning('Configured hook %s cannot be executed: %s',
                                  vendor_hook, e)
 
+        # Add additional metrics into the configuration according to the
+        # selected connection. We don't want to deal with overall config in the
+        # proxy, just pass required part.
+        if (self.config._influxdb_config
+                and 'additional_metric_tags' in self.config.config):
+            self.config._influxdb_config['additional_metric_tags'] = \
+                self.config.config['additional_metric_tags']
+
     @property
     def session(self):
         if not self._session:
@@ -458,7 +472,7 @@ class Connection(
         """
         # If we don't have a proxy, just instantiate Proxy so that
         # we get an adapter.
-        if isinstance(service, six.string_types):
+        if isinstance(service, str):
             service = service_description.ServiceDescription(service)
 
         # Directly invoke descriptor of the ServiceDescription
@@ -473,7 +487,7 @@ class Connection(
                 attr_name.replace('-', '_'),
                 property(fget=getter)
             )
-            self.config.enable_service(attr_name)
+        self.config.enable_service(service.service_type)
 
     def authorize(self):
         """Authorize this Connection
@@ -492,7 +506,14 @@ class Connection(
         try:
             return self.session.get_token()
         except keystoneauth1.exceptions.ClientException as e:
-            raise exceptions.raise_from_response(e.response)
+            raise exceptions.SDKException(e)
+
+    @property
+    def _pool_executor(self):
+        if not self.__pool_executor:
+            self.__pool_executor = concurrent.futures.ThreadPoolExecutor(
+                max_workers=5)
+        return self.__pool_executor
 
     def close(self):
         """Release any resources held open."""
