@@ -10,10 +10,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from openstack.network.v2 import _base
 from openstack import resource
 
 
-class Port(resource.Resource, resource.TagMixin):
+class Port(_base.NetworkResource, resource.TagMixin):
     resource_key = 'port'
     resources_key = 'ports'
     base_path = '/ports'
@@ -29,8 +30,9 @@ class Port(resource.Resource, resource.TagMixin):
     _query_mapping = resource.QueryParameters(
         'binding:host_id', 'binding:profile', 'binding:vif_details',
         'binding:vif_type', 'binding:vnic_type',
-        'description', 'device_id', 'device_owner', 'fixed_ips', 'ip_address',
-        'mac_address', 'name', 'network_id', 'status', 'subnet_id',
+        'description', 'device_id', 'device_owner', 'fixed_ips', 'id',
+        'ip_address', 'mac_address', 'name', 'network_id', 'status',
+        'subnet_id',
         is_admin_state_up='admin_state_up',
         is_port_security_enabled='port_security_enabled',
         project_id='tenant_id',
@@ -86,6 +88,9 @@ class Port(resource.Resource, resource.TagMixin):
     extra_dhcp_opts = resource.Body('extra_dhcp_opts', type=list)
     #: IP addresses for the port. Includes the IP address and subnet ID.
     fixed_ips = resource.Body('fixed_ips', type=list)
+    #: Read-only. The ip_allocation indicates when ports use deferred,
+    # immediate or no IP allocation.
+    ip_allocation = resource.Body('ip_allocation')
     #: The administrative state of the port, which is up ``True`` or
     #: down ``False``. *Type: bool*
     is_admin_state_up = resource.Body('admin_state_up', type=bool)
@@ -114,8 +119,6 @@ class Port(resource.Resource, resource.TagMixin):
     # (i.e.: minimum-bandwidth) and traits (i.e.: vnic-type, physnet)
     # requested by a port to Nova and Placement.
     resource_request = resource.Body('resource_request', type=dict)
-    #: Revision number of the port. *Type: int*
-    revision_number = resource.Body('revision_number', type=int)
     #: The IDs of any attached security groups.
     #: *Type: list of strs of the security group IDs*
     security_group_ids = resource.Body('security_groups', type=list)
