@@ -181,6 +181,70 @@ class TestCloudRegion(base.TestCase):
         self.assertEqual(1, cc.get_connect_retries('compute'))
         self.assertEqual(3, cc.get_connect_retries('baremetal'))
 
+    def test_rackspace_workaround(self):
+        # We're skipping loader here, so we have to expand relevant
+        # parts from the rackspace profile. The thing we're testing
+        # is that the project_id logic works.
+        cc = cloud_region.CloudRegion("test1", "DFW", {
+            'profile': 'rackspace',
+            'region_name': 'DFW',
+            'auth': {'project_id': '123456'},
+            'block_storage_endpoint_override': 'https://example.com/v2/',
+        })
+        self.assertEqual(
+            'https://example.com/v2/123456',
+            cc.get_endpoint('block-storage')
+        )
+
+    def test_rackspace_workaround_only_rax(self):
+        cc = cloud_region.CloudRegion("test1", "DFW", {
+            'region_name': 'DFW',
+            'auth': {'project_id': '123456'},
+            'block_storage_endpoint_override': 'https://example.com/v2/',
+        })
+        self.assertEqual(
+            'https://example.com/v2/',
+            cc.get_endpoint('block-storage')
+        )
+
+    def test_get_region_name(self):
+
+        def assert_region_name(default, compute):
+            self.assertEqual(default, cc.region_name)
+            self.assertEqual(default, cc.get_region_name())
+            self.assertEqual(default, cc.get_region_name(service_type=None))
+            self.assertEqual(
+                compute, cc.get_region_name(service_type='compute'))
+            self.assertEqual(
+                default, cc.get_region_name(service_type='placement'))
+
+        # No region_name kwarg, no regions specified in services dict
+        # (including the default).
+        cc = cloud_region.CloudRegion(config=fake_services_dict)
+        assert_region_name(None, None)
+
+        # Only region_name kwarg; it's returned for everything
+        cc = cloud_region.CloudRegion(
+            region_name='foo', config=fake_services_dict)
+        assert_region_name('foo', 'foo')
+
+        # No region_name kwarg; values (including default) show through from
+        # config dict
+        services_dict = dict(
+            fake_services_dict,
+            region_name='the-default', compute_region_name='compute-region')
+        cc = cloud_region.CloudRegion(config=services_dict)
+        assert_region_name('the-default', 'compute-region')
+
+        # region_name kwarg overrides config dict default (for backward
+        # compatibility), but service-specific region_name takes precedence.
+        services_dict = dict(
+            fake_services_dict,
+            region_name='dict', compute_region_name='compute-region')
+        cc = cloud_region.CloudRegion(
+            region_name='kwarg', config=services_dict)
+        assert_region_name('kwarg', 'compute-region')
+
     def test_aliases(self):
         services_dict = fake_services_dict.copy()
         services_dict['volume_api_version'] = 12
@@ -327,3 +391,20 @@ class TestCloudRegion(base.TestCase):
         cc = cloud_region.CloudRegion(
             "test1", "region-al", {}, auth_plugin=mock.Mock())
         self.assertIsNone(cc.get_session_endpoint('notfound'))
+
+    def test_get_endpoint_from_catalog(self):
+        dns_override = 'https://override.dns.example.com'
+        self.cloud.config.config['dns_endpoint_override'] = dns_override
+        self.assertEqual(
+            'https://compute.example.com/v2.1/',
+            self.cloud.config.get_endpoint_from_catalog('compute'))
+        self.assertEqual(
+            'https://internal.compute.example.com/v2.1/',
+            self.cloud.config.get_endpoint_from_catalog(
+                'compute', interface='internal'))
+        self.assertIsNone(
+            self.cloud.config.get_endpoint_from_catalog(
+                'compute', region_name='unknown-region'))
+        self.assertEqual(
+            'https://dns.example.com',
+            self.cloud.config.get_endpoint_from_catalog('dns'))
