@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import testtools
+from openstack.tests.unit import base
 
 from openstack.network.v2 import rbac_policy
 
@@ -24,17 +24,16 @@ EXAMPLE = {
 }
 
 
-class TestRBACPolicy(testtools.TestCase):
+class TestRBACPolicy(base.TestCase):
 
     def test_basic(self):
         sot = rbac_policy.RBACPolicy()
         self.assertEqual('rbac_policy', sot.resource_key)
         self.assertEqual('rbac_policies', sot.resources_key)
         self.assertEqual('/rbac-policies', sot.base_path)
-        self.assertEqual('network', sot.service.service_type)
         self.assertTrue(sot.allow_create)
-        self.assertTrue(sot.allow_get)
-        self.assertTrue(sot.allow_update)
+        self.assertTrue(sot.allow_fetch)
+        self.assertTrue(sot.allow_commit)
         self.assertTrue(sot.allow_delete)
         self.assertTrue(sot.allow_list)
 
